const constants = require('.././constants.json');

let log_channel = constants.log_channel,
    metrics = {  }

let kafka = require('kafka-node'),
    Producer = kafka.Producer,
    client = new kafka.KafkaClient({
        kafkaHost: constants.kafka_host,
        autoConnect: true
    }),
    producer = new Producer(client)

function collectMetrics(metric) {
    
    if (!(metric.functionHash in metrics)) {
        metrics[metric.functionHash] = {
            shortterm: {
                coldstart: 0,
                coldstart_total_request: 0,
                warm_total_request: 0,
                warmstart: 0,
                worker_count: 0
            }
        }
    }
    if (metric.type === 'coldstart') {
        metrics[metric.functionHash].shortterm.coldstart += metric.value
        metrics[metric.functionHash].shortterm.coldstart_total_request += 1
    } else if (metric.type === 'warmstart') {
        metrics[metric.functionHash].shortterm.warmstart += metric.value
        metrics[metric.functionHash].shortterm.warm_total_request += 1
    } else if (metric.type === 'scale') {
        metrics[metric.functionHash].worker_count = metric.value
    }
    
}

function broadcastMetrics() {
    
    if (Object.keys(metrics).length !== 0) {
        for (let [functionHash, metric] of Object.entries(metrics)) {
            
            if (metric.longterm === undefined) {
                metric.longterm = {
                        coldstart: 0,
                        coldstart_total_request: 0,
                        warm_total_request: 0,
                        warmstart: 0
                }
            }
            
            metric.longterm.coldstart = metric.longterm.coldstart 
                            * metric.longterm.coldstart_total_request
                            + metric.shortterm.coldstart
            
            metric.longterm.coldstart_total_request += metric.shortterm.coldstart_total_request
            metric.longterm.coldstart /= (metric.longterm.coldstart_total_request != 0)? 
                        metric.longterm.coldstart_total_request: 1

            metric.longterm.warmstart = metric.longterm.warmstart
                            * metric.longterm.warm_total_request
                            + metric.shortterm.warmstart
            metric.longterm.warm_total_request += metric.shortterm.warm_total_request
            metric.longterm.warmstart /= (metric.longterm.warm_total_request != 0)? 
                        metric.longterm.warm_total_request: 1

            metric.shortterm.coldstart /= (metric.shortterm.coldstart_total_request != 0)? 
                metric.shortterm.coldstart_total_request: 1
            metric.shortterm.warmstart /= (metric.shortterm.warm_total_request != 0)? 
                metric.shortterm.warm_total_request: 1
            metric.timestamp = Date.now()
        }
        
        let log = [{
            topic: log_channel,
            messages: JSON.stringify({
                metrics
            }),
            partition: 0
        }]
        producer.send(log, () => { })
        
        for (let [functionHash, metric] of Object.entries(metrics)) {
            metric.shortterm = {
                coldstart: 0,
                coldstart_total_request: 0,
                warm_total_request: 0,
                warmstart: 0,
                worker_count: 0
            }
        }
    }
}

module.exports = {
    collectMetrics, broadcastMetrics
}